<?php

/**
 *  @file
 *  Extends the MediaInternetBaseHandler class to handle TED videos.
 */

/**
 * Implementation of MediaInternetBaseHandler.
 *
 * @see hook_media_internet_providers().
 */
class MediaInternetTedHandler extends MediaInternetBaseHandler {
  public function parse($embedCode) {
    if (preg_match('@\[ted id=([0-9]+).*\]@i', $embedCode, $matches) && isset($matches[1])) {
      $id = $matches[1];
      $lang = NULL;

      if (preg_match('@lang=([a-z]{2}).*@i', $embedCode, $matches)) {
        $lang = $matches[1];
      }

      $embedCode = $this->getEmbedFromId($id, $lang);
    }
    $pattern = '@ted\.com\/(talks\/[^\'"]*)@i';

    if (preg_match($pattern, $embedCode, $matches) && isset($matches[1])) {
      return file_stream_wrapper_uri_normalize('ted://v/' . urlencode($matches[1]));
    }
  }

  public function getEmbedFromId($id, $language = NULL) {
    // @TODO use oembed (URL - http://www.ted.com/talks/1821?language=ro).
    $url = 'http://www.ted.com/talks/embed/id/'. $id;
    if ($language) {
      $url .= '/lang/'. check_plain($language);
    }

    $response = drupal_http_request($url);
    if ($response->code != 200) {
      throw new MediaInternetValidationException(t('The TED video ID is invalid or the video was deleted.'));
    }
    return $response->data;
  }

  public function claim($embedCode) {
    if ($this->parse($embedCode)) {
      return TRUE;
    }
  }

  public function save() {
    $file = $this->getFileObject();
    // If a user enters a duplicate Ted URL, the object will be saved again.
    // Set the timestamp to the current time, so that the media item shows up
    // at the top of the media library, where they would expect to see it.
    $file->timestamp = REQUEST_TIME;
    file_save($file);
    return $file;
  }

  public function getFileObject() {
    $uri = $this->parse($this->embedCode);
    $file = file_uri_to_object($uri, TRUE);

    // Try to default the file name to the video's title.
    if (empty($file->fid) && $info = $this->getOEmbed()) {
      $file->filename = truncate_utf8($info['title'], 255);
    }

    return $file;
  }

  /**
   * Returns information about the media. See http://www.oembed.com/.
   *
   * @return
   *   If oEmbed information is available, an array containing 'title', 'type',
   *   'url', and other information as specified by the oEmbed standard.
   *   Otherwise, NULL.
   */
  public function getOEmbed() {
    $uri = $this->parse($this->embedCode);

    $external_url = file_create_url($uri);
    $oembed_url = url('http://www.ted.com/services/v1/oembed.json', array(
      'query' => array('url' => $external_url))
    );
    $response = drupal_http_request($oembed_url);

    if (!isset($response->error)) {
      return drupal_json_decode($response->data);
    }
    else {
      throw new Exception("Error Processing Request. (Error: {$response->code}, {$response->error})");
      return;
    }
  }
}
