<?php

/**
 *  @file
 *  Create a Ted Stream Wrapper class for the Media/Resource module.
 */

/**
 *  Create an instance like this:
 *  $ted = new ResourceTedStreamWrapper('ted://v/[video-code]');
 */
class MediaTedStreamWrapper extends MediaReadOnlyStreamWrapper {
  protected $base_url = 'http://ted.com/';
  protected $oembed_base = 'http://www.ted.com/services/v1/oembed.json';

  function getTarget($f) {
    return FALSE;
  }

  static function getMimeType($uri, $mapping = NULL) {
    return 'video/ted';
  }

  function getExternalUrl() {
    $parts = $this->get_parameters();

    return $this->base_url . check_plain(urldecode($parts['v']));
  }

  function getOriginalThumbnailPath() {
    $parts = $this->get_parameters();
    $id = check_plain($parts['v']);
    $uri = file_stream_wrapper_uri_normalize('ted://v/' . $id);

    $oembed = $this->getOEmbed($uri);

    if (!empty($oembed['thumbnail_url'])) {
      return $oembed['thumbnail_url'];
    }
  }

  function getLocalThumbnailPath() {
    $parts = $this->get_parameters();
    $local_path = 'public://media-ted/' . check_plain($parts['v']) . '.jpg';

    if (!file_exists($local_path)) {
      $dirname = drupal_dirname($local_path);
      file_prepare_directory($dirname, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS);
      @copy($this->getOriginalThumbnailPath(), $local_path);
    }

    return $local_path;
  }

  /**
   * Returns oEmbed data.
   */
  public function getOEmbed($file_uri) {
    $external_url = file_create_url($file_uri);
    $oembed_url = url($this->oembed_base, array(
      'query' => array('url' => $external_url)
    ));
    $response = drupal_http_request($oembed_url);

    if (!isset($response->error)) {
      return drupal_json_decode($response->data);
    }
    else {
      throw new Exception("Error Processing Request. (Error: {$response->code}, {$response->error})");
      return;
    }
  }
}
